==> xdvik-22.74.1.tar.gz: xdvik-22.74.1/texk/xdvik/tfmload.c <==
/* This code was stolen from dvips for xdvi use.  The file itself did
 * not carry any licensing info.  According to other filed the code
 * either public domain, or, in the worst case, GPL.
 *
 * Loads a tfm file.  It marks the characters as undefined.
 *
 * Modified for use by xdvi/t1 by Nicolai Langfeldt */

#include <stdio.h>

#include "xdvi-config.h"
#include "xdvi.h"
#include "util.h"
#include "tfmload.h"

#ifdef T1LIB

#include "kpathsea/tex-file.h"

int fallbacktfm = 0;

Boolean
tfmload(const char *name, long *design, long *widths, long *fontdimen2)
{
    int i;
    int li;
    int nw, hd;
    int bc, ec;
    int f_param_off, f_param_num;
    long wtab[256];
    unsigned short chardat[256];
    char *filename;
    FILE *curfnt;

    fallbacktfm = 0;

    filename = kpse_find_tfm(name);

    if (filename == NULL) {
      filename = kpse_find_tfm("cmr10.tfm");
      if (fallbacktfm == 0)
	  fallbacktfm = 1;
    }

    if (filename == NULL)
	return False;

    curfnt = fopen(filename, "r");

    if (curfnt == NULL)
	return False;

    /* Next, we read the font data from the tfm file, and store it in
     * our own array.  */

    /* Lengths section, 16 bit unsigned quanities */
    li = two(curfnt);	/* lf = length of file, in words(4 bytes) */
    hd = two(curfnt);	/* lh = length of header data, in words */
    bc = two(curfnt);	/* bc = lowest charcode */
    ec = two(curfnt);	/* ec = largest charcode */
    nw = two(curfnt);	/* nw = number of width words */
    f_param_off = two(curfnt);	/* nh = number of height words */
    f_param_off += two(curfnt);	/* nd = number of depth words */
    f_param_off += two(curfnt);	/* ni = number of italic words */
    f_param_off += two(curfnt);	/* nl = number of words in lig/kern table */
    f_param_off += two(curfnt);	/* nk = number of words in the kern table */
    f_param_off += two(curfnt);	/* ne = number of words in the extensible char tab */
    f_param_num = two(curfnt);	/* np = number of font parameter words */

    /* Header, 32 bit signed quantities */
    four(curfnt);	/* header[0]: checksum */
    *design = four(curfnt);	/* header[1]: design size */

    /* Skip the rest: header[2..(hd-1)] */
    for (i = 2; i < hd; i++)
	li = four(curfnt);

    /* Initialize to undef value */
    for (i = 0; i < 256; i++) {
	chardat[i] = 256;
	widths[i] = 0;
    }

    /* The charinfo array: */
    for (i = bc; i <= ec; i++) {
	chardat[i] = one(curfnt);	/* 8 bits of width index */
	one(curfnt);	/* skip */
	two(curfnt);	/* skip, jump */
    }

    /* The (compressed!) width table */
    for (i = 0; i < nw; i++)
	wtab[i] = four(curfnt);

    /* skip to font parameter words */
    for (i = 0; i < f_param_off; i++)
	four(curfnt);

    /* read fontdimen2 (with of space) */
    for (i = 0; i < f_param_num; i++) {
	long l = four(curfnt);
	if (i == 1) {
	    *fontdimen2 = l;
	    if (debug & DBG_T1)
		fprintf(stderr, "fontdimen2 of %s: %ld\n", filename, *fontdimen2);
	}
	    
    }
    (void)fclose(curfnt);

    /* Now whe have the table and the indexes.  Expand into a directly
       mapped width array */
    for (i = bc; i <= ec; i++)
	if (chardat[i] != 256)
	    widths[i] = wtab[chardat[i]];

    return True;
}

#endif /* T1LIB */

==> xdvik-22.76.1.tar.gz: xdvik-22.76.1/texk/xdvik/tfmload.c <==
/* This code was stolen from dvips for xdvi use.  The file itself did
 * not carry any licensing info.  According to other filed the code
 * either public domain, or, in the worst case, GPL.
 *
 * Loads a tfm file.  It marks the characters as undefined.
 *
 * Modified for use by xdvi/t1 by Nicolai Langfeldt */

#include <stdio.h>

#include "xdvi-config.h"
#include "xdvi.h"
#include "util.h"
#include "tfmload.h"

#ifdef T1LIB

#include "kpathsea/tex-file.h"

int fallbacktfm = 0;

Boolean
tfmload(const char *name, long *design, long *widths, long *fontdimen2)
{
    int i;
    int li;
    int nw, hd;
    int bc, ec;
    int f_param_off, f_param_num;
    long wtab[256];
    unsigned short chardat[256];
    char *filename;
    FILE *curfnt;

    fallbacktfm = 0;

    filename = kpse_find_tfm(name);

    if (filename == NULL) {
      filename = kpse_find_tfm("cmr10.tfm");
      if (fallbacktfm == 0)
	  fallbacktfm = 1;
    }

    if (filename == NULL)
	return False;

    curfnt = XFOPEN(filename, "r");

    if (curfnt == NULL)
	return False;

    /* Next, we read the font data from the tfm file, and store it in
     * our own array.  */

    /* Lengths section, 16 bit unsigned quanities */
    li = two(curfnt);	/* lf = length of file, in words(4 bytes) */
    hd = two(curfnt);	/* lh = length of header data, in words */
    bc = two(curfnt);	/* bc = lowest charcode */
    ec = two(curfnt);	/* ec = largest charcode */
    nw = two(curfnt);	/* nw = number of width words */
    f_param_off = two(curfnt);	/* nh = number of height words */
    f_param_off += two(curfnt);	/* nd = number of depth words */
    f_param_off += two(curfnt);	/* ni = number of italic words */
    f_param_off += two(curfnt);	/* nl = number of words in lig/kern table */
    f_param_off += two(curfnt);	/* nk = number of words in the kern table */
    f_param_off += two(curfnt);	/* ne = number of words in the extensible char tab */
    f_param_num = two(curfnt);	/* np = number of font parameter words */

    /* Header, 32 bit signed quantities */
    four(curfnt);	/* header[0]: checksum */
    *design = four(curfnt);	/* header[1]: design size */

    /* Skip the rest: header[2..(hd-1)] */
    for (i = 2; i < hd; i++)
	li = four(curfnt);

    /* Initialize to undef value */
    for (i = 0; i < 256; i++) {
	chardat[i] = 256;
	widths[i] = 0;
    }

    /* The charinfo array: */
    for (i = bc; i <= ec; i++) {
	chardat[i] = one(curfnt);	/* 8 bits of width index */
	one(curfnt);	/* skip */
	two(curfnt);	/* skip, jump */
    }

    /* The (compressed!) width table */
    for (i = 0; i < nw; i++)
	wtab[i] = four(curfnt);

    /* skip to font parameter words */
    for (i = 0; i < f_param_off; i++)
	four(curfnt);

    /* read fontdimen2 (with of space) */
    for (i = 0; i < f_param_num; i++) {
	long l = four(curfnt);
	if (i == 1) {
	    *fontdimen2 = l;
	    if (debug & DBG_T1)
		fprintf(stderr, "fontdimen2 of %s: %ld\n", filename, *fontdimen2);
	}
	    
    }
    (void)fclose(curfnt);

    /* Now whe have the table and the indexes.  Expand into a directly
       mapped width array */
    for (i = bc; i <= ec; i++)
	if (chardat[i] != 256)
	    widths[i] = wtab[chardat[i]];

    return True;
}

#endif /* T1LIB */
